#include "X3D_TrimmedSurface.h"
#include "X3D_Contour2D.h"
#include "X3D_NurbsSurface.h"

#include <algorithm>
#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

TrimmedSurface::TrimmedSurface()
  : X3DParametricGeometryNode(), _surface(0)
{
  defineTypeName("TrimmedSurface", "NURBS");
}

TrimmedSurface::TrimmedSurface(NurbsSurface *surface)
  : X3DParametricGeometryNode(), _surface(0)
{
  defineTypeName("TrimmedSurface", "NURBS");
  
  setSurface(static_cast<NurbsSurface *>(surface));
}

TrimmedSurface::TrimmedSurface(const TrimmedSurface &N)
  : X3DParametricGeometryNode(N), _surface(0)
{
  for (MFNode::const_iterator it = N._trimmingContour.begin(); it != N._trimmingContour.end(); ++it)
    setChild(*it);
  
  setSurface(static_cast<NurbsSurface *>(N._surface));
}

SFAbstractNode TrimmedSurface::clone() const
{
  return new TrimmedSurface(*this);
}

TrimmedSurface::~TrimmedSurface()
{
  for (MFNode::const_iterator it = _trimmingContour.begin(); it != _trimmingContour.end(); ++it)
    removeParentFromChild(this, *it);
    
  removeParentFromChild(this, _surface);
}

MFAbstractNode TrimmedSurface::getChildList() const
{
  MFAbstractNode childrenList;
  for (MFNode::const_iterator it = _trimmingContour.begin(); it != _trimmingContour.end(); ++it)
    if (*it != 0)
      childrenList.push_back(*it);
  
  if (_surface != 0)
    childrenList.push_back(_surface);
    
  return childrenList; 
}

void TrimmedSurface::setSurface(NurbsSurface *surface)
{
  removeParentFromChild(this, _surface);
  _surface = surface;
  addParentToChild(this, _surface);
}

bool TrimmedSurface::addChild(const SFAbstractNode &N)
{
  if (dynamic_cast<Contour2D *>(N) != 0)
  {
    MFNode::iterator res = find(_trimmingContour.begin(), _trimmingContour.end(), static_cast<SFNode>(N));
    if (res == _trimmingContour.end())
    {
      _trimmingContour.push_back(static_cast<SFNode>(N));
      addParentToChild(this, N);
      return true;
    }
    return false;
  }
  if (dynamic_cast<NurbsSurface *>(N) != 0)
  {
      if (_surface != 0)
        return false;
      
      setSurface(static_cast<NurbsSurface *>(N));
      return true;    
  }
  if (N != 0)
    cx3d << "X3D::TrimmedSurface::addChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  else
    cx3d << "X3D::TrimmedSurface::addChild : cannot add a NULL node!" << endl;
   
  
  return false;
}

bool TrimmedSurface::setChild(const SFAbstractNode &N)
{
  if (dynamic_cast<Contour2D *>(N) != 0)
  {
    MFNode::iterator res = find(_trimmingContour.begin(), _trimmingContour.end(), static_cast<SFNode>(N));
    if (res == _trimmingContour.end())
    {
      _trimmingContour.push_back(static_cast<SFNode>(N));
      addParentToChild(this, N);
      return true;
    }
    return false;
  }  

  if (dynamic_cast<NurbsSurface *>(N) != 0)
  {
      setSurface(static_cast<NurbsSurface *>(N));
      return true;    
  }
  if (N != 0)
    cx3d << "X3D::TrimmedSurface::setChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  else
    cx3d << "X3D::TrimmedSurface::setChild : cannot add a NULL node!" << endl;
   
  
  return false;    
}

bool TrimmedSurface::removeChild(const SFAbstractNode &N)
{
  MFNode::iterator res = find(_trimmingContour.begin(), _trimmingContour.end(), N);
  if (res != _trimmingContour.end())
  {
    _trimmingContour.erase(res);
    removeParentFromChild(N, this);
    return true;
  }

  if (_surface == N)
  {
    setSurface(0);
    return true;
  } 

  return false;
}

}
}
